import logging
import requests
import ari

logging.basicConfig(level=logging.ERROR)

client = ari.connect('http://localhost:8088', 'asterisk', 'asterisk')

def safe_hangup(channel):
    """Safely hang up the specified channel"""
    try:
        channel.hangup()
        print "Hung up {}".format(channel.json.get('name'))
    except requests.HTTPError as e:
        if e.response.status_code != 404:
            raise e

def safe_bridge_destroy(bridge):
    """Safely destroy the specified bridge"""
    try:
        bridge.destroy()
    except requests.HTTPError as e:
        if e.response.status_code != 404:
            raise e

def teardown_bridge(channel, outgoing, bridge):
    """Tear down bridge"""
    safe_hangup(outgoing)
    safe_bridge_destroy(bridge)
    safe_hangup(channel)

def stasis_start_cb(channel_obj, ev):
    """Handler for StasisStart"""

    channel = channel_obj.get('channel')
    channel_name = channel.json.get('name')
    args = ev.get('args')

    if not args:
        print "Error: {} didn't provide any arguments!".format(channel_name)
        return

    if args and args[0] != 'inbound':
        # Only handle inbound channels here
        print "Non inbound channel {} entered stasis".format(channel_name)
        return

    if len(args) != 2:
        print "Error: {} didn't tell us who to dial".format(channel_name)
        channel.hangup()
        return

    print "{} entered our application".format(channel_name)

    channel.answer()

    outgoing = client.channels.create(
        endpoint=args[1],
        app='bridge-dial',
        appArgs='dialed')

    bridge = client.bridges.create(type='mixing')
    bridge.addChannel(channel=[channel.id, outgoing.id])

    channel.on_event('StasisEnd', lambda *args: teardown_bridge(channel, outgoing, bridge))
    outgoing.on_event('StasisEnd', lambda *args: teardown_bridge(channel, outgoing, bridge))

    def outgoing_dial_cb(channel_obj, ev):
        """Channel answered handler for our dialed channel"""
        print "dial event: {}".format(ev['dialstatus'])
        if ev['dialstatus'] == 'ANSWER':
            channel.answer()

    outgoing.on_event('Dial', outgoing_dial_cb)

    print "Dialing {}".format(args[1])
    outgoing.dial(timeout=10)

client.on_channel_event('StasisStart', stasis_start_cb)
client.run(apps='bridge-dial')